@@ -8,7 +8,7 @@
 #'   \code{"svm_rbf"}, \code{"nearest_neighbor"}, \code{"naive_Bayes"}, \code{"mlp"},
 #'   \code{"deep_learning"}, \code{"discrim_linear"}, \code{"discrim_quad"}, \code{"bag_tree"},
 #'   \code{"elastic_net"}, \code{"bayes_glm"}, \code{"pls"}, \code{"linear_reg"},
-#'   \code{"ridge_regression"}, and \code{"lasso_regression"}.
+#'   \code{"ridge_reg"}, \code{"lasso_reg"}, and \code{"penalized_cox"}.
 #'
 #' @param task A character string specifying the task type, typically \code{"classification"} or \code{"regression"}.
 #'
@@ -25,9 +25,11 @@
 #'     \item For \code{"xgboost"} and \code{"lightgbm"}, default values are provided for parameters like tree depth, learning rate, and sample size.
 #'     \item For \code{"logistic_reg"} and \code{"multinom_reg"}, the function returns defaults for regularization parameters (\code{penalty} and \code{mixture}) that vary with the specified engine.
 #'     \item For \code{"decision_tree"}, the parameters (such as \code{tree_depth}, \code{min_n}, and \code{cost_complexity}) are set based on the engine (e.g., \code{"rpart"}, \code{"C5.0"}, \code{"partykit"}, \code{"spark"}).
-#'     \item Other algorithms, including \code{"svm_linear"}, \code{"svm_rbf"}, \code{"nearest_neighbor"}, \code{"naive_Bayes"}, \code{"mlp"}, \code{"deep_learning"}, \code{"elastic_net"}, \code{"bayes_glm"}, \code{"pls"}, \code{"linear_reg"}, \code{"ridge_regression"}, and \code{"lasso_regression"}, have their respective default parameter lists.
+#'     \item Other algorithms, including \code{"svm_linear"}, \code{"svm_rbf"}, \code{"nearest_neighbor"}, \code{"naive_Bayes"}, \code{"mlp"}, \code{"deep_learning"}, \code{"elastic_net"}, \code{"bayes_glm"}, \code{"pls"}, \code{"linear_reg"}, \code{"ridge_reg"}, and \code{"lasso_reg"}, have their respective default parameter lists.
 #'   }
 #'
+#'@importFrom utils tail
+#'
 #' @export
 get_default_params <- function(algo, task, num_predictors = NULL, engine = NULL) {
   switch(algo,
@@ -331,12 +333,16 @@ get_default_params <- function(algo, task, num_predictors = NULL, engine = NULL)
          # 22. Linear Regression
          "linear_reg" = list(),
          # 23. Ridge Regression
-         "ridge_regression" = list(
+         "ridge_reg" = list(
            penalty = 0.01,
            mixture = 0
          ),
          # 24. Lasso Regression
-         "lasso_regression" = list(
+         "lasso_reg" = list(
+           penalty = 0.01,
+           mixture = 1
+         ),
+         "penalized_cox" = list(
            penalty = 0.01,
            mixture = 1
          ),
@@ -347,7 +353,7 @@ get_default_params <- function(algo, task, num_predictors = NULL, engine = NULL)
 #'
 #' Returns a list of default tuning parameter ranges for a specified algorithm based on the provided training data, outcome label, and engine.
 #'
-#' @param algo A character string specifying the algorithm name. Supported values include: \code{"rand_forest"}, \code{"C5_rules"}, \code{"xgboost"}, \code{"lightgbm"}, \code{"logistic_reg"}, \code{"multinom_reg"}, \code{"decision_tree"}, \code{"svm_linear"}, \code{"svm_rbf"}, \code{"nearest_neighbor"}, \code{"naive_Bayes"}, \code{"mlp"}, \code{"deep_learning"}, \code{"discrim_linear"}, \code{"discrim_quad"}, \code{"bag_tree"}, \code{"elastic_net"}, \code{"bayes_glm"}, \code{"pls"}, \code{"linear_reg"}, \code{"ridge_regression"}, and \code{"lasso_regression"}.
+#' @param algo A character string specifying the algorithm name. Supported values include: \code{"rand_forest"}, \code{"C5_rules"}, \code{"xgboost"}, \code{"lightgbm"}, \code{"logistic_reg"}, \code{"multinom_reg"}, \code{"decision_tree"}, \code{"svm_linear"}, \code{"svm_rbf"}, \code{"nearest_neighbor"}, \code{"naive_Bayes"}, \code{"mlp"}, \code{"deep_learning"}, \code{"discrim_linear"}, \code{"discrim_quad"}, \code{"bag_tree"}, \code{"elastic_net"}, \code{"bayes_glm"}, \code{"pls"}, \code{"linear_reg"}, \code{"ridge_reg"}, and \code{"lasso_reg"}.
 #'
 #' @param train_data A data frame containing the training data.
 #'
@@ -522,12 +528,12 @@ get_default_tune_params <- function(algo, train_data, label, engine) {
          "linear_reg" = NULL,
 
          # 23. Ridge Regression
-         "ridge_regression" = list(
+         "ridge_reg" = list(
            penalty = c(-5, 0)  # log scale
          ),
 
          # 24. Lasso Regression
-         "lasso_regression" = list(
+         "lasso_reg" = list(
            penalty = c(-5, 0)  # log scale
          ),
 
