@@ -79,14 +79,32 @@ plot.fastml <- function(x,
   positive_class   <- x$positive_class
   engine_names     <- x$engine_names
 
-  # Rebuild performance_wide (same logic as in summary.fastml)
+  # Rebuild performance_wide (robust to single- or multi-engine structures)
   metrics_list <- lapply(names(performance), function(model_name) {
-    engine_dfs <- lapply(names(performance[[model_name]]), function(engine_name) {
-      df <- as.data.frame(performance[[model_name]][[engine_name]])
-      df$Engine <- engine_name
-      df
-    })
-    combined_engines      <- do.call(rbind, engine_dfs)
+    perf_entry <- performance[[model_name]]
+
+    resolve_engine <- function(default_engine = NA_character_) {
+      if (!is.null(engine_names) && !is.null(engine_names[[model_name]]) && !is.na(engine_names[[model_name]])) {
+        return(engine_names[[model_name]])
+      }
+      if (!is.null(best_model_name) && model_name %in% names(best_model_name)) {
+        return(as.character(best_model_name[[model_name]]))
+      }
+      default_engine
+    }
+
+    if (is.list(perf_entry) && !is.data.frame(perf_entry)) {
+      engine_dfs <- lapply(names(perf_entry), function(engine_name) {
+        df <- as.data.frame(perf_entry[[engine_name]])
+        df$Engine <- engine_name
+        df
+      })
+      combined_engines <- do.call(rbind, engine_dfs)
+    } else {
+      combined_engines <- as.data.frame(perf_entry)
+      combined_engines$Engine <- resolve_engine()
+    }
+
     combined_engines$Model <- model_name
     combined_engines
   })
@@ -102,8 +120,13 @@ plot.fastml <- function(x,
 
   if (task == "classification") {
     desired_metrics <- c("accuracy", "f_meas", "kap", "precision", "sens", "spec", "roc_auc")
-  } else {
+  } else if (task == "regression") {
     desired_metrics <- c("rmse", "rsq", "mae")
+  } else if (task == "survival") {
+    brier_metrics <- sort(grep("^brier_t", all_metric_names, value = TRUE))
+    desired_metrics <- c("c_index", "uno_c", "ibs", "rmst_diff", brier_metrics)
+  } else {
+    desired_metrics <- unique(performance_df$.metric)
   }
   desired_metrics <- intersect(desired_metrics, all_metric_names)
   if (length(desired_metrics) == 0) {
@@ -123,24 +146,23 @@ plot.fastml <- function(x,
     ) %>%
       dplyr::select(Model, Engine, accuracy, kap, sens, spec, precision, f_meas)
   } else {
-    if (task == "classification") {
-      performance_wide <- tidyr::pivot_wider(
-        performance_sub,
-        names_from  = .metric,
-        values_from = .estimate
-      ) %>%
-        dplyr::select(Model, Engine, accuracy, kap, sens, spec, precision, f_meas, roc_auc)
-    } else {
-      performance_wide <- tidyr::pivot_wider(
-        performance_sub,
-        names_from  = .metric,
-        values_from = .estimate
-      ) %>%
-        dplyr::select(Model, Engine, rmse, rsq, mae)
+    keep_metrics <- desired_metrics
+    if (length(engine_names) == 1 && "LiblineaR" %in% engine_names) {
+      keep_metrics <- intersect(keep_metrics, c("accuracy", "kap", "sens", "spec", "precision", "f_meas"))
     }
+    performance_wide <- tidyr::pivot_wider(
+      performance_sub,
+      names_from  = .metric,
+      values_from = .estimate
+    )
+    select_cols <- c("Model", "Engine", keep_metrics)
+    select_cols <- intersect(select_cols, colnames(performance_wide))
+    performance_wide <- dplyr::select(performance_wide, dplyr::all_of(select_cols))
   }
 
-  if (task == "regression" && main_metric != "rsq") {
+  brier_cols <- grep("^brier_t", colnames(performance_wide), value = TRUE)
+  ascending_metrics <- unique(c("rmse", "mae", "ibs", "logloss", "mse", brier_cols))
+  if (main_metric %in% ascending_metrics) {
     performance_wide <- performance_wide[order(performance_wide[[main_metric]], na.last = TRUE), ]
   } else {
     performance_wide <- performance_wide[order(-performance_wide[[main_metric]], na.last = TRUE), ]
@@ -156,8 +178,33 @@ plot.fastml <- function(x,
     spec      = "Specificity",
     rsq       = "R-squared",
     mae       = "MAE",
-    rmse      = "RMSE"
+    rmse      = "RMSE",
+    c_index   = "Harrell C-index",
+    uno_c     = "Uno's C-index",
+    ibs       = "Integrated Brier Score",
+    rmst_diff = "RMST diff (t_max)"
   )
+  t_max_val <- x$survival_t_max
+  if (!is.null(t_max_val) && length(t_max_val) == 1 && is.finite(t_max_val) && t_max_val > 0) {
+    display_names[["rmst_diff"]] <- sprintf(
+      "RMST diff (t<=%s)",
+      format(t_max_val, trim = TRUE, digits = 4)
+    )
+  }
+  if (!is.null(x$survival_brier_times)) {
+    for (nm in names(x$survival_brier_times)) {
+      if (!is.null(x$survival_brier_times[[nm]]) && is.finite(x$survival_brier_times[[nm]])) {
+        time_label <- format(x$survival_brier_times[[nm]], trim = TRUE, digits = 4)
+      } else {
+        time_label <- nm
+      }
+      display_names[[nm]] <- sprintf("Brier(t=%s)", time_label)
+    }
+  }
+  auto_brier_cols <- setdiff(grep("^brier_t", colnames(performance_wide), value = TRUE), names(display_names))
+  if (length(auto_brier_cols) > 0) {
+    display_names[auto_brier_cols] <- auto_brier_cols
+  }
 
   # ============================
   # 1. Bar plot of metrics
@@ -188,6 +235,18 @@ plot.fastml <- function(x,
       if (length(present_reg_metrics) > 0) {
         performance_melt$Metric <- factor(performance_melt$Metric, levels = present_reg_metrics)
       }
+    } else if (task == "survival") {
+      rmst_label <- display_names[["rmst_diff"]]
+      if (is.null(rmst_label)) {
+        rmst_label <- "RMST diff (t_max)"
+      }
+      surv_order <- c("Harrell C-index", "Uno's C-index", "Integrated Brier Score", rmst_label)
+      brier_labels <- display_names[grep("^brier_t", names(display_names))]
+      surv_order <- c(surv_order, brier_labels)
+      present_surv_metrics <- intersect(surv_order, unique(performance_melt$Metric))
+      if (length(present_surv_metrics) > 0) {
+        performance_melt$Metric <- factor(performance_melt$Metric, levels = present_surv_metrics)
+      }
     }
 
     p_bar <- ggplot2::ggplot(
