@@ -6,19 +6,26 @@
 #'
 #' @param object A fitted `fastml` object created by the `fastml()` function.
 #' @param newdata A data frame or tibble containing new predictor data for which to generate predictions.
-#' @param type Type of prediction to return. One of `"auto"` (default), `"class"`, `"prob"`, or `"numeric"`.
-#'   - `"auto"`: chooses `"class"` for classification and `"numeric"` for regression.
+#' @param type Type of prediction to return. One of `"auto"` (default), `"class"`, `"prob"`, `"numeric"`,
+#'   `"survival"`, or `"risk"`.
+#'   - `"auto"`: chooses `"class"` for classification, `"numeric"` for regression, and `"survival"` for survival.
 #'   - `"prob"`: returns class probabilities (only for classification).
 #'   - `"class"`: returns predicted class labels.
 #'   - `"numeric"`: returns predicted numeric values (for regression).
+#'   - `"survival"`: returns survival probabilities at the supplied `eval_time` horizons (for survival tasks).
+#'   - `"risk"`: returns risk scores on the linear predictor scale (for survival tasks).
 #' @param model_name (Optional) Name of a specific model to use when `object$best_model` contains multiple models.
 #' @param verbose Logical; if `TRUE`, prints progress messages showing which models are used during prediction.
 #' @param postprocess_fn (Optional) A function to apply to the final predictions (e.g., inverse transforms, thresholding).
+#' @param eval_time Optional numeric vector of time points (on the original time
+#'   scale) at which to return survival probabilities when `type = "survival"`.
+#'   Required for survival tasks when requesting survival curves.
 #' @param ... Additional arguments (currently unused).
 #'
 #' @return A vector of predictions, or a named list of predictions (if multiple models are used).
 #'         If `postprocess_fn` is supplied, its output will be returned instead.
 #'
+#' @method predict fastml
 #' @importFrom stats predict
 #' @importFrom tibble is_tibble
 #' @export
@@ -44,6 +51,7 @@ predict.fastml <- function(object, newdata,
                            model_name = NULL,
                            verbose = FALSE,
                            postprocess_fn = NULL,
+                           eval_time = NULL,
                            ...) {
   # 1. input checks & drop label from newdata -----------------------------
   if (missing(newdata)) {
@@ -53,8 +61,11 @@ predict.fastml <- function(object, newdata,
   if (is.null(lbl)) {
     stop("Label name is missing from the model object.")
   }
-  if (lbl %in% names(newdata)) {
-    newdata[[lbl]] <- NULL
+  label_cols <- unique(as.character(lbl))
+  label_cols <- label_cols[label_cols %in% names(newdata)]
+  if (length(label_cols)) {
+    keep_cols <- setdiff(names(newdata), label_cols)
+    newdata <- newdata[, keep_cols, drop = FALSE]
   }
 
   # 2. preprocessing via recipe -------------------------------------------
@@ -71,13 +82,15 @@ predict.fastml <- function(object, newdata,
 
   # 3. pick prediction type ------------------------------------------------
   predict_type <- switch(type,
-                         auto = if (object$task == "classification") "class" else "numeric",
+                         auto = if (object$task == "classification") "class" else if (object$task == "survival") "survival" else "numeric",
                          prob = {
                            if (object$task != "classification") {
                              warning("Probabilities only for classification; using numeric.")
                              "numeric"
                            } else "prob"
                          },
+                         survival = "survival",
+                         risk = "risk",
                          type
   )
 
@@ -111,7 +124,69 @@ predict.fastml <- function(object, newdata,
   preds <- lapply(names(to_predict), function(nm) {
     wf <- to_predict[[nm]]
     if (verbose) message("Predicting with: ", nm)
-    p <- predict(wf, new_data = new_proc, type = predict_type, ...)
+
+    new_data_for_predict <- if (inherits(wf, "workflow")) newdata else new_proc
+
+    if (object$task == "survival") {
+      if (identical(predict_type, "survival")) {
+        times <- eval_time
+        if (is.null(times)) {
+          stop("Please supply 'eval_time' when requesting survival probabilities.")
+        }
+        times <- as.numeric(times)
+        times <- sort(unique(times[is.finite(times) & times >= 0]))
+        if (length(times) == 0) {
+          stop("No valid evaluation times supplied in 'eval_time'.")
+        }
+        surv_pred <- tryCatch({
+          if (inherits(wf, "fastml_native_survival")) {
+            predict_survival(wf, newdata = newdata, times = times, ...)
+          } else {
+            predict_survival(wf, newdata = newdata, times = times, ...)
+          }
+        }, error = function(e) {
+          stop(sprintf("Failed to obtain survival predictions for model '%s': %s", nm, e$message), call. = FALSE)
+        })
+        if (is.matrix(surv_pred) && ncol(surv_pred) == length(times)) {
+          colnames(surv_pred) <- format(times, trim = TRUE, scientific = FALSE)
+        }
+        if (!is.null(postprocess_fn)) {
+          surv_pred <- postprocess_fn(surv_pred)
+        }
+        return(surv_pred)
+      }
+
+      if (identical(predict_type, "risk")) {
+        risk_pred <- tryCatch({
+          if (inherits(wf, "fastml_native_survival")) {
+            predict_risk(wf, newdata = new_data_for_predict, ...)
+          } else {
+            pred_obj <- predict(wf, new_data = new_data_for_predict, type = "linear_pred", ...)
+            if (is.data.frame(pred_obj) || tibble::is_tibble(pred_obj)) {
+              if (".pred" %in% names(pred_obj)) {
+                pred_obj$.pred
+              } else if (".pred_lp" %in% names(pred_obj)) {
+                pred_obj$.pred_lp
+              } else {
+                as.numeric(pred_obj[[1]])
+              }
+            } else {
+              as.numeric(pred_obj)
+            }
+          }
+        }, error = function(e) {
+          stop(sprintf("Failed to obtain risk predictions for model '%s': %s", nm, e$message), call. = FALSE)
+        })
+        if (!is.null(postprocess_fn)) {
+          risk_pred <- postprocess_fn(risk_pred)
+        }
+        return(as.numeric(risk_pred))
+      }
+
+      stop("Unsupported prediction type for survival task: ", predict_type)
+    }
+
+    p <- predict(wf, new_data = new_data_for_predict, type = predict_type, ...)
     # pull out the vector (or keep full prob‐tibble)
     if (is.data.frame(p) || tibble::is_tibble(p)) {
       p <- switch(predict_type,
