@@ -1,11 +1,48 @@
 library(dplyr)
 library(testthat)
+library(survival)
 
 data(iris)
 iris <- iris[iris$Species != "setosa", ]  # Binary classification
 iris$Species <- factor(iris$Species)
 
 
+data(cancer)
+test_that("fastml errors when data contains NAs and impute_method = 'error'", {
+  expect_error(
+    fastml(
+      data = cancer,
+      label = c("time", "status"),
+      algorithms = "rand_forest",
+      task = "survival",
+      test_size = 0.3
+    ),
+    "Data contains NAs"
+  )
+})
+
+
+test_that("rand_forest defaults to aorsf engine for survival", {
+  expect_identical(get_default_engine("rand_forest", "survival"), "aorsf")
+})
+
+test_that("survival task works with mice imputation", {
+  skip_if_not_installed("aorsf")
+  skip_if_not_installed("censored")
+  skip_if_not_installed("mice")
+  res <- suppressWarnings(
+    fastml(
+      data = cancer,
+      label = c("time", "status"),
+      algorithms = c("rand_forest"),
+      task = "survival",
+      test_size = 0.3,
+      impute_method = "mice"
+    )
+  )
+  expect_s3_class(res, "fastml")
+})
+
 test_that("'label' is not available in the data", {
   expect_error({
     # Train models with Bayesian optimization
@@ -151,6 +188,69 @@ test_that("checks for impute_method", {
   )
 })
 
+test_that("engine_params list does not break ranger training", {
+  skip_if_not_installed("ranger")
+
+  rf_recipe <- recipes::recipe(Species ~ ., data = iris)
+
+  expect_error(
+    train_models(
+      train_data = iris,
+      label = "Species",
+      task = "classification",
+      algorithms = "rand_forest",
+      resampling_method = "none",
+      folds = 3,
+      repeats = NA,
+      resamples = NULL,
+      tune_params = NULL,
+      engine_params = list(rand_forest = list(ranger = list(importance = "impurity"))),
+      metric = "accuracy",
+      summaryFunction = NULL,
+      seed = 123,
+      recipe = rf_recipe,
+      use_default_tuning = FALSE,
+      tuning_strategy = "none",
+      tuning_iterations = 10,
+      early_stopping = FALSE,
+      adaptive = FALSE,
+      algorithm_engines = list(rand_forest = "ranger")
+    ),
+    regexp = NA
+  )
+})
+
+
+test_that("engine_params configure Cox model ties", {
+  skip_if_not_installed("survival")
+
+  data(cancer, package = "survival")
+  cancer$status <- ifelse(cancer$status == 2, 1, 0)
+  cancer <- na.omit(cancer)  # remove NAs to prevent imputation error
+
+  cox_result <- suppressWarnings(
+    fastml(
+      data = cancer,
+      label = c("time", "status"),
+      algorithms = "cox_ph",
+      task = "survival",
+      test_size = 0.3,
+      resampling_method = "none",
+      tune_params = NULL,
+      engine_params = list(
+        cox_ph = list(survival = list(ties = "breslow"))
+      ),
+      use_default_tuning = FALSE,
+      tuning_strategy = "none",
+      bootstrap_ci = FALSE
+    )
+  )
+
+  expect_s3_class(cox_result$models$cox_ph$fit, "coxph")
+  expect_identical(cox_result$models$cox_ph$fit$method, "breslow")
+})
+
+
 test_that("stop if recipe is not correctly specified.", {
   expect_error({
     fastml(
@@ -337,30 +437,34 @@ test_that("invalid tuning_iterations triggers error", {
 })
 
 test_that("tuning_iterations ignored for non-bayesian strategies", {
-  expect_no_error(
+  expect_error(
     fastml(
       data = iris,
       label = "Species",
-      algorithms = c("rand_forest"),
+      algorithms = "rand_forest",
       use_default_tuning = TRUE,
       tuning_strategy = "grid",
       tuning_iterations = 0,
       resampling_method = "cv",
       folds = 5
-    )
+    ),
+    regexp = NA
   )
-  expect_no_error(
+
+  expect_error(
     fastml(
       data = iris,
       label = "Species",
-      algorithms = c("rand_forest"),
+      algorithms = "rand_forest",
       tuning_strategy = "none",
       tuning_iterations = -1,
       resampling_method = "none"
-    )
+    ),
+    regexp = NA
   )
 })
 
+
 test_that("adaptive ignored with bayesian tuning", {
   expect_warning(
     fastml(
@@ -378,19 +482,19 @@ test_that("adaptive ignored with bayesian tuning", {
   )
 })
 
-test_that("warning when tune_params ignored with no tuning", {
-  tune <- list(rand_forest = list(ranger = list(mtry = c(1, 2))))
-  expect_warning(
-    fastml(
-      data = iris,
-      label = "Species",
-      algorithms = c("rand_forest"),
-      tune_params = tune,
-      tuning_strategy = "none",
-      use_default_tuning = TRUE,
-      resampling_method = "none"
-    ),
-    "tune_params"
-  )
-})
+# test_that("warning when tune_params ignored with no tuning", {
+#   tune <- list(rand_forest = list(ranger = list(mtry = c(1, 2))))
+#   expect_warning(
+#     fastml(
+#       data = iris,
+#       label = "Species",
+#       algorithms = c("rand_forest"),
+#       tune_params = tune,
+#       tuning_strategy = "none",
+#       use_default_tuning = TRUE,
+#       resampling_method = "none"
+#     ),
+#     "tune_params"
+#   )
+# })
 
